﻿Imports System.Windows.Forms
Imports System.DirectoryServices.ActiveDirectory
Imports System.DirectoryServices

Friend Class AdTreeViewForm


#Region "Private Objects"

    Private _DomainController As String = String.Empty

#End Region


#Region "Properties"

    Private _SelectedContainerPath As String = String.Empty
    Public ReadOnly Property SelectedContainerPath() As String
        Get
            Return _SelectedContainerPath
        End Get
    End Property

    Private _Username As String = String.Empty
    Public WriteOnly Property Username() As String
        Set(ByVal value As String)
            _Username = value
        End Set
    End Property

    Private _Password As String = String.Empty
    Public WriteOnly Property Password() As String
        Set(ByVal value As String)
            _Password = value
        End Set
    End Property

    Private _DomainName As String = String.Empty
    Public Property DomainName() As String
        Get
            Return _DomainName
        End Get
        Set(ByVal value As String)
            _DomainName = value
        End Set
    End Property

#End Region


#Region "Methods"

    Private Sub InitialLoad()
        Try
            If String.IsNullOrEmpty(_DomainName) Then
                Me.DomainName = ActiveDirectoryHelper.GetLocalDomainName
            End If
            If String.IsNullOrEmpty(_DomainController) Then
                _DomainController = ActiveDirectoryHelper.GetSingleDomainController(_DomainName, _Username, _Password)
            End If

            Dim RootNode As New AdTreeNode
            Dim DomainObject As Domain = Domain.GetDomain(ActiveDirectoryHelper.GetDomainContext(_DomainName, _Username, _Password))
            Using RootDirectoryEntry As DirectoryEntry = DomainObject.GetDirectoryEntry
                RootNode.DistinguishedName = CStr(RootDirectoryEntry.Properties("distinguishedName").Value).Replace("/", "\/")
            End Using
            RootNode.DisplayName = Me.DomainName
            RootNode.Type = AdTreeNode.AdObjectType.Domain
            InitialLoadFinished(RootNode, Nothing)
        Catch ex As Exception
            InitialLoadFinished(Nothing, ex.Message)
        End Try
    End Sub

    Private Sub InitialLoadFinished(ByVal RootNode As AdTreeNode, ByVal ErrorMessage As String)
        If Me.InvokeRequired Then
            Me.Invoke(New Action(Of AdTreeNode, String)(AddressOf InitialLoadFinished), RootNode, ErrorMessage)
        Else
            If Not Me.IsDisposed Then
                Try
                    If String.IsNullOrEmpty(ErrorMessage) Then
                        RootNode.Nodes.Add(New AdTreeNode With {.DisplayName = "Loading..."})
                        DomainTreeView.Nodes.Add(RootNode)
                        Dim bgThread As New System.Threading.Thread(AddressOf LoadAdNodes)
                        bgThread.Start(RootNode)
                    Else
                        MessageBox.Show("The domain tree load could not be completed due to the following error: " & ErrorMessage & _
                                        vbNewLine & "Verify that the domain name specified is correct (and is a fully qualified DNS domain name) and the credentials used are valid.", "Error Encountered During Domain Tree Load", MessageBoxButtons.OK, MessageBoxIcon.Error)
                        Me.DialogResult = Windows.Forms.DialogResult.Abort
                    End If
                Finally
                    EnableForm()
                End Try
            End If
        End If
    End Sub

    Private Sub LoadAdNodes(ByVal RootNodeObject As Object)
        Dim RootNode As AdTreeNode = DirectCast(RootNodeObject, AdTreeNode)
        Dim Children As New List(Of AdTreeNode)
        Try
            Using RootDirectoryEntry As New DirectoryEntry("LDAP://" & _DomainController & ":389/" & RootNode.DistinguishedName)
                RootDirectoryEntry.AuthenticationType = AuthenticationTypes.Secure
                If Not String.IsNullOrEmpty(_Username) Then
                    RootDirectoryEntry.Username = _Username
                    RootDirectoryEntry.Password = _Password
                End If
                For Each ChildObject As DirectoryEntry In RootDirectoryEntry.Children
                    Try
                        Dim ChildObjectType As String = ChildObject.SchemaClassName
                        If ChildObjectType = "organizationalUnit" OrElse ChildObjectType = "container" Then
                            Dim ChildNode As New AdTreeNode
                            ChildNode.DisplayName = CStr(ChildObject.Properties("name").Value)
                            If ChildObjectType = "organizationalUnit" Then
                                ChildNode.Type = AdTreeNode.AdObjectType.OU
                            ElseIf ChildObjectType = "container" Then
                                ChildNode.Type = AdTreeNode.AdObjectType.Container
                            Else
                                ChildNode.Type = AdTreeNode.AdObjectType.Unknown
                            End If
                            ChildNode.DistinguishedName = CStr(ChildObject.Properties("distinguishedName").Value).Replace("/", "\/")
                            For Each ChildChildObject As DirectoryEntry In ChildObject.Children
                                Try
                                    If ChildChildObject.SchemaClassName = "organizationalUnit" OrElse ChildChildObject.SchemaClassName = "container" Then
                                        ChildNode.Nodes.Add(New AdTreeNode With {.DisplayName = "Loading..."})
                                        Exit For
                                    End If
                                Finally
                                    ChildChildObject.Dispose()
                                End Try
                            Next
                            Children.Add(ChildNode)
                        End If
                    Finally
                        ChildObject.Dispose()
                    End Try
                Next
            End Using
            Children.Sort()
            LoadFinished(True, Nothing, DirectCast(RootNodeObject, AdTreeNode), Children)
        Catch ex As Exception
            LoadFinished(False, "The following error was encountered whilst trying to load the domain objects: " & ex.Message, Nothing, Nothing)
        End Try
    End Sub

    Private Sub LoadFinished(ByVal Success As Boolean, ByVal Message As String, ByVal Node As AdTreeNode, ByVal NewChildren As List(Of AdTreeNode))
        If Me.InvokeRequired Then
            Me.Invoke(New Action(Of Boolean, String, AdTreeNode, List(Of AdTreeNode))(AddressOf LoadFinished), Success, Message, Node, NewChildren)
        Else
            If Not Me.IsDisposed Then
                Try
                    If Success Then
                        Node.Nodes.Clear()
                        For i As Integer = 0 To NewChildren.Count - 1
                            Node.Nodes.Add(NewChildren(i))
                        Next
                    Else
                        MessageBox.Show(Message, "Error Encountered During Load", MessageBoxButtons.OK, MessageBoxIcon.Warning)
                    End If
                Catch ex As Exception
                    MessageBox.Show("Error adding completed node to tree: " & ex.Message, "Error Encountered During Node Add", MessageBoxButtons.OK, MessageBoxIcon.Warning)
                Finally
                    EnableForm()
                    StatusLbl.Text = "Ready"
                End Try
            End If
        End If
    End Sub

    Private Sub DisableForm()
        TitleLbl.Enabled = False
        DomainTreeView.Enabled = False
        ProgBar.Visible = True
        OKBtn.Enabled = False
        ContainerLbl.Enabled = False
    End Sub

    Private Sub EnableForm()
        TitleLbl.Enabled = True
        DomainTreeView.Enabled = True
        ProgBar.Visible = False
        OKBtn.Enabled = True
        ContainerLbl.Enabled = True
    End Sub

#End Region


#Region "Event Handlers"

    Private Sub AdTreeViewForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        DisableForm()
        Try
            AdImages.Images.Add("OrgImage", ActiveDirectoryIcon.GetIcon(ActiveDirectoryIcon.ActiveDirectoryIconType.OU))
            AdImages.Images.Add("DomainImage", ActiveDirectoryIcon.GetIcon(ActiveDirectoryIcon.ActiveDirectoryIconType.Domain))
            AdImages.Images.Add("ContainerImage", ActiveDirectoryIcon.GetIcon(ActiveDirectoryIcon.ActiveDirectoryIconType.Container))
        Catch ex As Exception
            MessageBox.Show("The following error was encountered whilst attempting to load domain/container/OU icons: " & ex.Message, "Error Loading Icons", MessageBoxButtons.OK, MessageBoxIcon.Error)
            StatusLbl.Text = "Error encountered"
            EnableForm()
        End Try

        Dim bgThread As New Threading.Thread(AddressOf InitialLoad)
        bgThread.IsBackground = True
        bgThread.Start()
    End Sub

    Private Sub DomainTreeView_BeforeExpand(sender As Object, e As System.Windows.Forms.TreeViewCancelEventArgs) Handles DomainTreeView.BeforeExpand
        Try
            Dim ExpandedNode As AdTreeNode = DirectCast(e.Node, AdTreeNode)
            If ExpandedNode.Nodes.Count > 0 Then
                StatusLbl.Text = "Loading child objects..."
                DisableForm()
                Dim bgThread As New System.Threading.Thread(AddressOf LoadAdNodes)
                bgThread.Start(ExpandedNode)
            End If
        Catch ex As Exception
            MessageBox.Show("Internal Error - Failed to get required information from expanded container node: " & ex.Message, "Container Expand Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
            EnableForm()
            StatusLbl.Text = "Error encountered"
        End Try
    End Sub

    Private Sub CancelBtn_Click(sender As System.Object, e As System.EventArgs) Handles CancelBtn.Click
        Me.DialogResult = Windows.Forms.DialogResult.Cancel
    End Sub

    Private Sub DomainTreeView_AfterSelect(sender As System.Object, e As System.Windows.Forms.TreeViewEventArgs) Handles DomainTreeView.AfterSelect
        SelectedContainerBox.Text = DirectCast(DomainTreeView.SelectedNode, AdTreeNode).DistinguishedName
    End Sub

    Private Sub OKBtn_Click(sender As System.Object, e As System.EventArgs) Handles OKBtn.Click
        If DomainTreeView.SelectedNode Is Nothing Then
            MessageBox.Show("Please select a container before clicking OK", "No Container Selected", MessageBoxButtons.OK, MessageBoxIcon.Warning)
        Else
            _SelectedContainerPath = DirectCast(DomainTreeView.SelectedNode, AdTreeNode).DistinguishedName
            Me.DialogResult = Windows.Forms.DialogResult.OK
        End If
    End Sub

#End Region



  

    
End Class